#pragma once

#include "data.hpp"

struct vec_istream: std::streambuf {
    vec_istream(std::vector<char> &vec) {
        this->setg(&vec[0], &vec[0], &vec[0] + vec.size());
    }
};

namespace net {

class PacketWriter final {
public:
    PacketWriter(size_t size, enet_uint32 flags): p(enet_packet_create(NULL, size, flags)) { }

    void write(const void* data, size_t size) {
        assert(i + size <= p->dataLength);
        memcpy(p->data + i, data, size);
        i += size;
    }
    template<typename D>
    void write(const D& d) {
        write(&d, sizeof(d));
    }

    bool isFull() const {
        return i >= p->dataLength;
    }

    packet_t *get() { return p; }

private:
    packet_t *p;
    size_t i = 0;
};

struct data_ref {
    void* d;
    size_t s;

    void *data() { return d; }
    const void *data() const { return d; }
    size_t size() const { return s; }
};

class PacketReader final {
public:
    PacketReader(packet_t* ptr, size_t offset): p(ptr), i(offset) { }
    PacketReader(packet_t* ptr, bool broadcasted = false):
        PacketReader(ptr, sizeof(enet_uint8) + (broadcasted ? 0 : sizeof(salt_t))) { }

    template<typename D>
    const D* read() {
        if (i + sizeof(D) > p->dataLength)
            return nullptr;

        auto ptr = (const D *)(p->data + i);
        i += sizeof(D);
        return ptr;
    }
    template<typename D>
    bool read(D& out) {
        const auto ptr = read<D>();
        if (ptr == nullptr)
            return false;

        out = *ptr;
        return true;
    }

    bool skip(size_t size) {
        i += size;
        return i < p->dataLength;
    }

    data_ref remaning() {
        auto res = data_ref{p->data + i, p->dataLength - i};
        i = p->dataLength;
        return res;
    }

    bool isFull() const {
        return i >= p->dataLength;
    }

    packet_t *get() { return p; }

private:
    packet_t *p;
    size_t i = 0;
};

}