#pragma once

#include <string>
#include <shared_mutex>
#include <fstream>
#include "../../../core/world/forward.h"
#include "../../../core/utils/zctx.hpp"
#include "../../../core/data/math.hpp"

namespace world::server {
    ///Group of chunks saved as a single file in memory
    class MemoryRegion {
    public:
        MemoryRegion(const std::string& folderPath, const area_<region_pos> &pos);
        ~MemoryRegion();

        typedef std::vector<char> data;

        bool read(const region_chunk_pos &pos, const zstd::read_ctx& ctx, data &out);
        void write(const region_chunk_pos &pos, const zstd::write_ctx& ctx, const std::string_view &in);

    private:
        void save(bool force = true);

        std::string path;
        //TODO: use tickets to remove unused regions

        std::shared_mutex mutex;
        robin_hood::unordered_map<region_chunk_pos, data*> content;
        bool changed = false;

        void load();
    };
}